import { HttpClient } from '@angular/common/http';
import { Component, ElementRef, Inject, Injector, OnDestroy, OnInit, ViewChild, ViewEncapsulation } from '@angular/core';
import { Router } from '@angular/router';
import { EventTooltipConfig, PresetManager, ResourceTimeRangeStore, SchedulerPro, SchedulerProjectModel, TimeRangesConfig } from '@bryntum/schedulerpro';
import { zip } from 'rxjs';
import { GanttCustomElementsInitializer } from './shared/gantt-custom-elements-initializer';


@Component({
  selector: 'app-scheduler',
  templateUrl: './scheduler.component.html',
  styleUrls: ['./scheduler.component.scss'],
  encapsulation: ViewEncapsulation.None
})
export class SchedulerComponent implements OnInit, OnDestroy {
  @ViewChild('schedulerDiv', { static: true }) private schedulerDiv!: ElementRef;

  private projectStart = new Date(2021, 8, 22);
  private projectEnd = new Date(2021, 10, 12);

  private schedulerInstance!: SchedulerPro;

  constructor(
    private readonly http: HttpClient,
    private readonly router: Router,
    @Inject(Injector) injector: Injector
  ) {
    GanttCustomElementsInitializer.iniitalize(injector);
  }

  ngOnInit(): void {
    this.removePresets();
    this.renderGantt();
    this.loadProject();
  }

  ngOnDestroy(): void {
    this.schedulerInstance.destroy();
  }

  private removePresets(): void {
    PresetManager.remove('weekAndDayLetter');
    PresetManager.remove('manyYears');
    PresetManager.remove('manyYears-80by50');
    PresetManager.remove('year-30by100');
    PresetManager.remove('year');
    PresetManager.remove('year-200by100');
    PresetManager.remove('year-50by100');
    PresetManager.remove('weekDateAndMonth');
    PresetManager.remove('weekAndMonth');
    PresetManager.remove('monthAndYear');
    PresetManager.remove('dayAndWeek');
    PresetManager.remove('minuteAndHour-30by60');
    PresetManager.remove('minuteAndHour-60by60');
    PresetManager.remove('minuteAndHour-100by60');
    PresetManager.remove('secondAndMinute-130by40');
    PresetManager.remove('secondAndMinute-60by40');
    PresetManager.remove('secondAndMinute');
  }

  private renderGantt(): void {
    const schedulerInstance = new SchedulerPro({
      appendTo: this.schedulerDiv.nativeElement,
      columns: [
        {
          type: 'resourceInfo',
          field: 'name',
          text:'Resource',
          minWidth: 200,
          showEventCount: true,
          showImage: false
        }
      ],
      features: {
        resourceTimeRanges: true,
        timeRanges: <TimeRangesConfig> {
          showCurrentTimeLine: {
            name: 'Now'
          },
          disabled: false,
          showHeaderElements: true
        }
      },
      createEventOnDblClick: false,
      zoomKeepsOriginalTimespan: true,
      rowHeight: 60,
      readOnly: false,
    });

    this.schedulerInstance = schedulerInstance;

    this.schedulerInstance.setTimeSpan(this.projectStart, this.projectEnd);


    this.schedulerInstance.on('beforeEventDropFinalize', async (event: any) => {
      event.context.async = true;

      await this.sleep(2000);

      event.context.finalize(true);
    })
  }

  private sleep(ms: number) {
    return new Promise(resolve => setTimeout(resolve, ms));
  }


  private loadProject(): void {
    setTimeout(() => {
      zip(
        this.http.get('../assets/json/resources.json'),
        this.http.get('../assets/json/assignments.json'),
        this.http.get('../assets/json/tasks.json')
      ).subscribe(
        (values: any[]) => this.onLoadProject(values),
        (err: any) => console.error(err));
    }, 100);
  }

  private async onLoadProject(values: any[]): Promise<void> {
    // 0 resources
    // 1 assignments
    // 2 tasks

    const project = new SchedulerProjectModel({
      resourcesData: <any>values[0],
      assignmentsData: <any>values[1],
      eventsData: <any>values[2],
    });

    this.schedulerInstance.project = project;

    this.setSchedulerTimelineRanges(this.projectStart, this.projectEnd);
    this.loadRanges();
  }

  private setSchedulerTimelineRanges(startDate: Date, endDate: Date): void {
    this.schedulerInstance.timeRanges.push(<any> {
      id: 'projectstart',
      name:'Start',
      startDate: startDate
    });

    this.schedulerInstance.timeRanges.push(<any> {
      id: 'projectend',
      name: 'End',
      startDate: endDate
    });
  }

  private loadRanges(): void {
    setTimeout(() => {
      this.http.get('../assets/json/resources-not-working-time.json').subscribe(
        (resourcesWorkingTime: any) => this.onLoadRanges(resourcesWorkingTime)
      );
    }, 1000);
  }

  private onLoadRanges(data: any[]): void {
    for(const interval of data) {
      const ganttResource  = this.schedulerInstance.resources.find((x: any) => x.id === interval.resourceId);

      (<ResourceTimeRangeStore>this.schedulerInstance.resourceTimeRangeStore).add({
        name           : 'New time range',
        startDate      : new Date(interval.startDate),
        endDate        : new Date(interval.endDate),
        timeRangeColor : 'green',
        resourceId     : (<any>ganttResource).id
      });
    }
  }
}
